-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
--  Hammer of Justas
--
-- Copyright (C) 2010-2020
-- By Jonathan E. Wright
--
-- FILE: HoJ_UI.lua
-- DESC: Addon user interface.

local addonName, HoJ = ...

-- DEFAULT_CHAT_FRAME:AddMessage( "HoJ_UI.lua" )

local SCROLLBAR_WIDTH = 16
local SCROLLBAR_HEIGHT = 12
local EDGE_SIZE = 8

local sliderBackdrop  = {
	bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
	edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
	tile = true, tileSize = 8, edgeSize = 8,
	insets = { left = 3, right = 3, top = 6, bottom = 6 }
};

local listBackdrop = {
  -- path to the background texture
  bgFile = "",
  -- path to the border texture
  edgeFile = "Interface/Buttons/UI-SliderBar-Border",
  -- true to repeat the background texture to fill the frame, false to scale it
  tile = true,
  -- size (width or height) of the square repeating background tiles (in pixels)
  tileSize = 8,
  -- thickness of edge segments and square size of edge corners (in pixels)
  edgeSize = EDGE_SIZE,
  -- distance from the edges of the frame to those of the background texture (in pixels)
  insets = {
	left = 3,
	right = 3,
	top = 3,
	bottom = 3
  }
};

-- -----------------------------------------------------------------------------
-- HoJ:AddonOptions_OnOK
-- -----------------------------------------------------------------------------
function HoJ:AddonOptions_OnOK()
	HoJ:CreateSavedVariablesFromHoJ( "Changes saved." )
end

-- -----------------------------------------------------------------------------
-- HoJ:AddonOptions_OnCancel
-- -----------------------------------------------------------------------------
function HoJ:AddonOptions_OnCancel()
	HoJ:RestoreSavedVariables()
end

-- -----------------------------------------------------------------------------
-- HoJ:AddonOptions_OnDefault
-- -----------------------------------------------------------------------------
function HoJ:AddonOptions_OnDefault()
	--HoJ:DebugMsg( "AddonOptions_OnDefault" );
end

-- -----------------------------------------------------------------------------
-- HoJ:AddonOptions_OnRefresh
-- -----------------------------------------------------------------------------
function HoJ:AddonOptions_OnRefresh()
	--HoJ:DebugMsg( "AddonOptions_OnRefresh" );
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleShowSpellOnBar
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowSpellOnBar( spell )
	spell.showOnBar = not spell.showOnBar
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleShowSkillSFrame
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowSkillSFrame()
	if ( HoJ._showCooldownsFrame ) then
		--HoJ:DebugMsg( "_showCooldownsFrame = false" );
		HoJ._showCooldownsFrame = false;
	else
		--HoJ:DebugMsg( "_showCooldownsFrame = true" );
		HoJ._showCooldownsFrame = true;
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleShowBuffsFrame
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowBuffsFrame()
	if ( HoJ._showBuffsFrame ) then
		--HoJ:DebugMsg( "_showBuffsFrame = false" );
		HoJ._showBuffsFrame = false;
	else
		--HoJ:DebugMsg( "_showBuffsFrame = true" );
		HoJ._showBuffsFrame = true;
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleShowRotationFrame
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowRotationFrame()
	HoJ._showRotationFrame = not HoJ._showRotationFrame
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleShowRotationSecondaryFrame
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowRotationSecondaryFrame()
	HoJ._showRotationSecondaryFrame = not HoJ._showRotationSecondaryFrame
end

-- -----------------------------------------------------------------------------
-- HoJ:TogglehowTemplarsVerdictFrame
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowTemplarsVerdictFrame()
	if ( HoJ._showTemplarsVerdictFrame ) then
		HoJ._showTemplarsVerdictFrame = false;
	else
		HoJ._showTemplarsVerdictFrame = true;
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleShowHolyPowerFrame
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowHolyPowerFrame()
	if ( HoJ._showHolyPowerFrame ) then
		HoJ._showHolyPowerFrame = false;
	else
		HoJ._showHolyPowerFrame = true;
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleEnableForSpec
-- -----------------------------------------------------------------------------
function HoJ:ToggleEnableForSpec( className, specName ) 
	local enabled = HoJ[className][specName]._enabled
	HoJ[className][specName]._enabled = not enabled
end

-- -----------------------------------------------------------------------------
-- HoJ:ToggleShowDebugFrame
-- -----------------------------------------------------------------------------
function HoJ:ToggleShowDebugFrame()
	HoJ._showDebugFrame = not HoJ._showDebugFrame
	HoJ:DebugMsg( "_showDebugFrame = " .. tostring( HoJ._showDebugFrame ) )
end

-- -----------------------------------------------------------------------------
-- HoJ:SetupCheckBox
-- -----------------------------------------------------------------------------
function HoJ:SetupCheckBox( parentFrame, anchorFrame, name, anchor1, anchor2, xoffset, yoffset, checked, fontSize, text )
	local cb = CreateFrame( "CheckButton", name, parentFrame, "OptionsCheckButtonTemplate" )
	cb:SetPoint( anchor1, anchorFrame, anchor2, xoffset, yoffset )
	--cb:SetScript( "OnClick", function ( self ) HoJ:ToggleShowSpellOnBar( spellInfo ) end )
	cb:SetChecked( checked )
	-- the following is really pretty stupid and was a PITA to figure out
	--getglobal( cb:GetName() .. 'Text' ):SetFont( "Fonts\\FRIZQT__.TTF", fontSize )
	getglobal( cb:GetName() .. 'Text' ):SetText( text )
	return cb;
end

-- -----------------------------------------------------------------------------
-- HoJ:GetCheckWidthWithText
-- -----------------------------------------------------------------------------
function HoJ:GetCheckWidthWithText( cb)
	local textWidth = getglobal( cb:GetName() .. 'Text' ):GetWidth()
	return textWidth + cb:GetWidth()
end

-- -----------------------------------------------------------------------------
-- HoJ:SetupSpellCheckBox
-- -----------------------------------------------------------------------------
function HoJ:SetupSpellCheckBox( parentFrame, anchorFrame, name, anchor1, anchor2, xoffset, yoffset, fontSize, spellInfo, text )
	local cb = CreateFrame( "CheckButton", name, parentFrame, "OptionsCheckButtonTemplate" );
	cb:SetPoint( anchor1, anchorFrame, anchor2, xoffset, yoffset );
	cb:SetScript( "OnClick", function ( self ) HoJ:ToggleShowSpellOnBar( spellInfo ) end );
	cb:SetChecked( spellInfo.showOnBar );
	--getglobal( cb:GetName() .. 'Text' ):SetFont( "Fonts\\FRIZQT__.TTF", fontSize );
	-- the following is really pretty stupid and was a PITA to figure out
	if ( spellInfo.spellType == HoJ.SPELL_BUFF or spellInfo.spellType == HoJ.SPELL_TALENT_BUFF ) then
		getglobal( cb:GetName() .. 'Text' ):SetText( text .. " (Buff)" );
	elseif ( spellInfo.spellType == HoJ.SPELL_SKILL ) then
		getglobal( cb:GetName() .. 'Text' ):SetText( text .. " (Skill)" );
	elseif ( spellInfo.spellType == HoJ.SPELL_TALENT ) then
		getglobal( cb:GetName() .. 'Text' ):SetText( text .. " (Talent)" );
	elseif ( spellInfo.spellType == HoJ.SPELL_SHAPESHIFT ) then
		getglobal( cb:GetName() .. 'Text' ):SetText( text .. " (Seal)" );
	else
		getglobal( cb:GetName() .. 'Text' ):SetText( text );
	end
	return cb;
end

-- -----------------------------------------------------------------------------
-- HoJ:SetupShowSpellCheckBox
-- -----------------------------------------------------------------------------
function HoJ:SetupShowSpellCheckBox( parentFrame, anchorFrame, name, anchor1, 
		anchor2, xoffset, yoffset, fontSize, spellInfo, count )
	return HoJ:SetupSpellCheckBox( parentFrame, anchorFrame, name, anchor1, 
		anchor2, xoffset, yoffset, fontSize, spellInfo, HoJ._text["SHOW"] .. spellInfo.name );
end

-- -----------------------------------------------------------------------------
-- HoJ:OnScrollValueChanged
-- -----------------------------------------------------------------------------
function HoJ:OnScrollValueChanged( slider, value )
	-- HoJ:DebugMsg( "OnScrollValueChanged(): slider = " .. slider.name .. ", value = " .. value )
	slider.scrollView:SetVerticalScroll( value )
	slider.scrollView:UpdateScrollChildRect()
	local itemFrame = slider.scrollView:GetScrollChild()
	-- HoJ:DebugMsg( "OnScrollValueChanged: itemFrame = " .. itemFrame:GetName() )
	itemFrame:SetPoint( "TOPLEFT", slider.scrollView, "TOPLEFT", 0, value )
	itemFrame:SetPoint( "TOPRIGHT", slider.scrollView, "TOPRIGHT", 0, value )
end

-- -----------------------------------------------------------------------------
-- scrollView_OnMouseWheel
-- -----------------------------------------------------------------------------
local function scrollView_OnMouseWheel( self, delta )
	local cur = self.slider:GetValue();
	local min, max = self.slider:GetMinMaxValues();
	local new = cur + ( delta * -1 * self.itemHeight );
	if ( new < min ) then
		new = min;
	elseif ( new > max ) then
		new = max;
	end
	self.slider:SetValue( new );
end

-- -----------------------------------------------------------------------------
-- Clamp
-- -----------------------------------------------------------------------------
 local function Clamp( value, min, max )
 	if ( value < min ) then
 		return min
 	elseif ( value > max ) then
 		return max
 	end

 	return value
 end

-- -----------------------------------------------------------------------------
-- HoJ:OnGlobalAlphaChanged
-- -----------------------------------------------------------------------------
function HoJ:OnGlobalAlphaChanged( alphaSlider, value )
	self._globalAlpha = Clamp( value / 10.0, 0.1, 1.0 )

	HoJFrame_Rotation:SetAlpha( self._globalAlpha * HoJ._rotationAlpha )
	HoJFrame_Cooldowns:SetAlpha( self._globalAlpha * HoJ._cooldownsAlpha )
	HoJFrame_Buffs:SetAlpha( self._globalAlpha * HoJ._buffsAlpha )
	-- holy power alpha is always set dynamically
	--HoJFrame_HoPo:SetAlpha( self._globalAlpha * HoJ._holyPowerAlpha )
	--HoJFrame:SetAlpha( self._globalAlpha * HoJ._holyPowerAlpha )

	--self:DebugMsg( "UI alpha = " .. self._globalAlpha )
end

-- -----------------------------------------------------------------------------
-- HoJ:OnRotationAlphaChanged
-- -----------------------------------------------------------------------------
function HoJ:OnRotationAlphaChanged( alphaSlider, value )
	self._rotationAlpha = Clamp( value / 10.0, 0.1, 1.0 )
	HoJFrame_Rotation:SetAlpha( self._globalAlpha * HoJ._rotationAlpha )
	--self:DebugMsg( "Rotation alpha = " .. self._rotationAlpha )
end

-- -----------------------------------------------------------------------------
-- HoJ:OnRotationSecondaryAlphaChanged
-- -----------------------------------------------------------------------------
function HoJ:OnRotationSecondaryAlphaChanged( alphaSlider, value )
	self._rotationSecondaryAlpha = Clamp( value / 10.0, 0.1, 1.0 )
	HoJ_RotationSecondaryFrame:SetAlpha( self._globalAlpha * HoJ._rotationSecondaryAlpha )
	--self:DebugMsg( "Rotation alpha = " .. self._rotationSecondaryAlpha )
end

-- -----------------------------------------------------------------------------
-- HoJ:OnCooldownsAlphaChanged
-- -----------------------------------------------------------------------------
function HoJ:OnCooldownsAlphaChanged( alphaSlider, value )
	self._cooldownsAlpha = Clamp( value / 10.0, 0.1, 1.0 )
	HoJFrame_Cooldowns:SetAlpha( self._globalAlpha * HoJ._cooldownsAlpha )
	--self:DebugMsg( "Cooldowns alpha = " .. self._cooldownsAlpha )
end

-- -----------------------------------------------------------------------------
-- HoJ:OnBuffsAlphaChanged
-- -----------------------------------------------------------------------------
function HoJ:OnBuffsAlphaChanged( alphaSlider, value )
	self._buffsAlpha = Clamp( value / 10.0, 0.1, 1.0 )
	HoJFrame_Buffs:SetAlpha( self._globalAlpha * HoJ._buffsAlpha )
	--self:DebugMsg( "Buffs alpha = " .. self._buffsAlpha )
end

-- -----------------------------------------------------------------------------
-- HoJ:OnHolyPowerAlphaChanged
-- -----------------------------------------------------------------------------
function HoJ:OnHolyPowerAlphaChanged( alphaSlider, value )
	self._holyPowerAlpha = Clamp( value / 10.0, 0.1, 1.0 )
	-- holy power alphas are updated dynamically each frame
	--HoJFrame_HoPo:SetAlpha( self._globalAlpha * HoJ._holyPowerAlpha )
	--HoJFrame:SetAlpha( self._globalAlpha * HoJ._holyPowerAlpha )
	--self:DebugMsg( "Holy Power alpha = " .. self._holyPowerAlpha )
end

-- -----------------------------------------------------------------------------
-- HoJ:OnDebugAlphaChanged
-- -----------------------------------------------------------------------------
function HoJ:OnDebugAlphaChanged( alphaSlider, value )
	self._debugAlpha = Clamp( value / 10.0, 0.1, 1.0 )
	HoJFrame_Debug:SetAlpha( self._globalAlpha * HoJ._debugAlpha )
	--self:DebugMsg( "value = " .. value .. ", debug alpha = " .. self._debugAlpha )
end

-- -----------------------------------------------------------------------------
-- HoJ:AddSlider
-- -----------------------------------------------------------------------------
function HoJ:AddSlider( name, title, parent, relativeFrame, relativeTo, ofsx, ofsy, width, height, min, max, fontSize, minText, maxText, value, callbackFn )
	local sl = CreateFrame( "Slider", name, parent, "OptionsSliderTemplate")
	--sl.name = "UI Alpha"
	sl:SetOrientation( "HORIZONTAL" )
	sl:SetHitRectInsets( 0, 0, -10, 0 )
	sl:SetBackdrop( sliderBackdrop )
	sl:SetThumbTexture( "Interface\\Buttons\\UI-SliderBar-Button-Horizontal" )
	sl.slider = sl
	sl:SetPoint( "TOPLEFT", relativeFrame, relativeTo, ofsx, ofsy )
	sl:SetWidth( width )
	sl:SetHeight( height )
	sl:SetScript( "OnValueChanged", callbackFn )
	sl:SetMinMaxValues( min, max )
	sl:SetValueStep( 1 )
	sl:SetValue( value )

	local lowText = getglobal( sl:GetName() .. "Low" )
	lowText:SetFont( "Fonts\\FRIZQT__.TTF", fontSize );
	lowText:SetText( minText )

	local highText = getglobal( sl:GetName() .. "High" )
	highText:SetFont( "Fonts\\FRIZQT__.TTF", fontSize );
 	highText:SetText( maxText )

	local titleText = getglobal( sl:GetName() .. "Text" )
	titleText:SetFont( "Fonts\\FRIZQT__.TTF", fontSize );
 	titleText:SetText( title )
 	return sl
 end

  -- -----------------------------------------------------------------------------
-- HoJ:AddHorizontalSlider
-- -----------------------------------------------------------------------------
function HoJ:AddHorizontalSlider( name, title, parent, relativeFrame, relativeTo, ofsx, ofsy, width, height, min, max, fontSize, minText, maxText, value, callbackFn )
	local sl = CreateFrame( "Slider", name, parent, "OptionsSliderTemplate")

	local titleText = getglobal( sl:GetName() .. "Text" )
	titleText:SetFont( "Fonts\\FRIZQT__.TTF", fontSize );
	titleText:SetText( title .. " : " .. maxText )
	--local titleTextWidth = titleText:GetWidth() + EDGE_SIZE
	local titleTextWidth = width / 2 - EDGE_SIZE
	local barWidth = width - titleTextWidth

	--sl.name = "UI Alpha"
	sl:SetOrientation( "HORIZONTAL" )
	sl:SetHitRectInsets( 0, 0, -10, 0 )
	sl:SetBackdrop( sliderBackdrop )
	sl:SetThumbTexture( "Interface\\Buttons\\UI-SliderBar-Button-Horizontal" )
	sl.slider = sl
	sl:SetWidth( barWidth )
	sl:SetHeight( height )
	sl:SetPoint( "TOPLEFT", relativeFrame, relativeTo, titleTextWidth + ofsx, ofsy )	
	sl:SetScript( "OnValueChanged", callbackFn )
	sl:SetMinMaxValues( min, max )
	sl:SetValueStep( 1 )
	sl:SetValue( value )

	titleText:ClearAllPoints()
	titleText:SetPoint( "TOPLEFT", sl, "TOPLEFT", -titleTextWidth, 0 )

	local lowText = getglobal( sl:GetName() .. "Low" )
	lowText:SetFont( "Fonts\\FRIZQT__.TTF", fontSize );
	lowText:SetText( minText )
	lowText:Hide()

	local highText = getglobal( sl:GetName() .. "High" )
	highText:SetFont( "Fonts\\FRIZQT__.TTF", fontSize )
	highText:SetText( maxText )
	highText:Hide()

 	return sl
 end

-- -----------------------------------------------------------------------------
-- HoJ:GetSliderSizeWithText
-- -----------------------------------------------------------------------------
 function HoJ:GetSliderSizeWithText( slider ) 
	local lowHeight = getglobal( slider:GetName() .. "Low" ):GetHeight()
	local highHeight = getglobal( slider:GetName() .. "High" ):GetHeight()
	local textHeight = getglobal( slider:GetName() .. "Text" ):GetHeight()
	local sliderHeight = slider:GetHeight()
	local sliderWidth = slider:GetWidth()
	
	local lowHighHeight = lowHeight
	if ( highHeight > lowHighHeight ) then
		lowHighHeight = highHeight
	end
	
	HoJ:DebugMsg( "sliderWidth = " .. sliderWidth .. ", sliderHeight = " .. sliderHeight .. ", lowHeight = " .. lowHeight .. ", highHeight = " .. highHeight )

	return sliderWidth, sliderHeight + lowHighHeight + textHeight
 end

 -- -----------------------------------------------------------------------------
-- HoJ:SetupCheckboxesForSpellInfos
-- -----------------------------------------------------------------------------
function HoJ:SetupCheckboxesForSpellInfos( spellInfos, checkFrame, prev, anchor, yofs, count )
	if ( spellInfos == nil ) then
		HoJ:Error( "Why you pass nil table, fool?" )
		return 0, nil, nil
	end

	local spellIndex = 0
	local startCount = count

	for key, value in pairs( spellInfos ) do
		if ( value.showCheckbox ) then
			--HoJ:DebugMsg( key .. " is " .. value.name )
			if ( key == nil ) then
				HoJ:DebugMsg( "got nil name for spellInfo id " .. value.id )
			elseif ( value == nil ) then
				HoJ:DebugMsg( "got nil value for spellInfo id " .. value.id )
			elseif ( value.name == nil ) then
				HoJ:DebugMsg( "got nil value.name for spellInfo id " .. value.id )
			else
				prev = HoJ:SetupShowSpellCheckBox( checkFrame, prev, key, "TOPLEFT", anchor, 0, yofs, 10, value, count )
				if ( count == startCount ) then
					anchor = "BOTTOMLEFT"
					yofs = 2
				end
			end
			count = count + 1
		end
		spellIndex = spellIndex + 1
	end

	return count, prev, anchor
end

-- -----------------------------------------------------------------------------
-- HoJ:AddHeadingToScrollList
-- -----------------------------------------------------------------------------
function HoJ:AddHeadingToScrollList( listFrame, prev, anchor, xofs, yofs, fontSize, count, text )
	heading = listFrame:CreateFontString( nil, "ARTWORK" )

	heading:SetFont( "Fonts\\FRIZQT__.TTF", fontSize )
	heading:SetJustifyH( "LEFT" )
	heading:SetJustifyV( "TOP" )
	heading:ClearAllPoints()
	heading:SetTextColor( 1, 1, 1, 1 )
	heading:SetPoint( "TOPLEFT", prev, anchor, 0, yofs )
	heading:SetText( text )
	
	count = count + 1

	return count, heading
end

-- -----------------------------------------------------------------------------
-- HoJ:AddTextToFrame
-- -----------------------------------------------------------------------------
function HoJ:AddTextToFrame( ownerFrame, parentFrame, textAnchor, parentAnchor, xofs, yofs, r, g, b, fontSize, text )
	local fs = ownerFrame:CreateFontString( nil, "ARTWORK" );
	fs:SetFont( "Fonts\\FRIZQT__.TTF", fontSize );
	fs:SetJustifyH( "LEFT" );
	fs:SetJustifyV( "TOP" );
	fs:ClearAllPoints();
	fs:SetTextColor( r, g, b, 1 );
	fs:SetPoint( textAnchor, parentFrame, parentAnchor, xofs, yofs );
	fs:SetText( text );
	return fs
end

-- -----------------------------------------------------------------------------
-- HoJ:SetupAddOnOptions
-- -----------------------------------------------------------------------------
function HoJ:SetupAddOnOptions()
	--HoJ:DebugMsg( "HoJ:SetupAddOnOptions" );
	local panel = CreateFrame( "FRAME", "HoJOptions" );
	--panel.name = "Hammer of Justas";
	panel.name = GetAddOnMetadata( "HammerOfJustas", "Title" );
	panel.parent = nil;
	panel.okay = function ( self ) HoJ:AddonOptions_OnOK( self ) end;
	panel.cancel = function ( self ) HoJ:AddonOptions_OnCancel( self ) end;
	panel.defalt = function ( self ) HoJ:AddonOptions_OnDefault( self ) end;
	panel.refresh = function ( self ) HoJ:AddonOptions_OnRefresh( self ) end;
	InterfaceOptions_AddCategory( panel );

	local panelHeader = panel:CreateFontString( nil, "ARTWORK" );
	panelHeader:SetFont( "Fonts\\FRIZQT__.TTF", 24 );
	panelHeader:SetJustifyH( "LEFT" );
	panelHeader:SetJustifyV( "TOP" );
	panelHeader:ClearAllPoints();
	panelHeader:SetPoint( "TOPLEFT", 16, -16 );
	panelHeader:SetText( "Hammer of Justas v. " .. HoJ:GetVersionString() );

	------------------------------------------------------------
	-- paladin skills list
	------------------------------------------------------------
	local paladinChecklistTitle = panel:CreateFontString( nil, "ARTWORK" )
	paladinChecklistTitle:SetFont( "Fonts\\FRIZQT__.TTF", 12 )
	paladinChecklistTitle:SetJustifyH( "LEFT" )
	paladinChecklistTitle:SetJustifyV( "TOP" )
	paladinChecklistTitle:ClearAllPoints()
	paladinChecklistTitle:SetTextColor( 1, 1, 1, 1 )
	paladinChecklistTitle:SetPoint( "TOPLEFT", panelHeader, "BOTTOMLEFT", 0, -16 )
	paladinChecklistTitle:SetText( HoJ._text["Paladin Settings"] )

	local SCROLL_BORDER_HEIGHT = 24 * 8 + EDGE_SIZE * 2;
	local paladinScrollBorder = CreateFrame( "FRAME", "paladinScrollBorder", panel );
	paladinScrollBorder:SetPoint( "TOPLEFT", paladinChecklistTitle, "BOTTOMLEFT", 0, 0 );
	--paladinScrollBorder:SetPoint( "TOPRIGHT", panel, "TOP", -EDGE_SIZE, 0 );
	paladinScrollBorder:SetPoint( "BOTTOMRIGHT", panel, "CENTER", -EDGE_SIZE, 0 );
	paladinScrollBorder:SetBackdrop( listBackdrop );
	paladinScrollBorder:Show()

	local paladinScrollView = CreateFrame( "SCROLLFRAME", "paladinScrollView", paladinScrollBorder );
	paladinScrollView:EnableMouseWheel( true );
	paladinScrollView:SetScript( "OnMouseWheel", scrollView_OnMouseWheel );
	paladinScrollView:SetPoint( "TOPLEFT", paladinScrollBorder, "TOPLEFT", EDGE_SIZE, -EDGE_SIZE );
	paladinScrollView:SetPoint( "TOPRIGHT", paladinScrollBorder, "TOPRIGHT", -EDGE_SIZE, -EDGE_SIZE );
	paladinScrollView:SetPoint( "BOTTOMLEFT", paladinScrollBorder, "BOTTOMLEFT", EDGE_SIZE,
	EDGE_SIZE );
	paladinScrollView:SetPoint( "BOTTOMRIGHT", paladinScrollBorder, "BOTTOMRIGHT", -EDGE_SIZE, EDGE_SIZE );

	local paladinCheckFrame = CreateFrame( "FRAME", "paladinCheckFrame", paladinScrollView );
	paladinScrollView:SetScrollChild( paladinCheckFrame );

	paladinCheckFrame:SetPoint( "TOPLEFT", paladinScrollView, "TOPLEFT", 0, 0 );
	paladinCheckFrame:Lower();

	local YPAD = 2
	local count = 0
	local prev = nil

	count, prev = HoJ:AddHeadingToScrollList( paladinCheckFrame, paladinCheckFrame, "TOPLEFT", 0, 0, 10, count, HoJ._text["Frame Toggles"] )

	local checkShowTemplarsVerdictFrame = HoJ:SetupCheckBox( paladinCheckFrame, prev, 
			"checkShowTemplarsVerdict", "TOPLEFT", "BOTTOMLEFT", 0, 0, 
			HoJ._showTemplarsVerdictFrame, 10, HoJ._text["Show Templars Verdict Frame"] )
	checkShowTemplarsVerdictFrame:SetScript( "OnClick", function ( self ) HoJ:ToggleShowTemplarsVerdictFrame() end )
	checkShowTemplarsVerdictFrame:Show()

	local checkShowHolyPowerFrame = HoJ:SetupCheckBox( paladinCheckFrame, checkShowTemplarsVerdictFrame, 
			"checkShowHolyPower", "TOPLEFT", "BOTTOMLEFT", 0, 0, 
			HoJ._showHolyPowerFrame, 10, HoJ._text["Show Holy Power Frame"] )
	checkShowHolyPowerFrame:SetScript( "OnClick", function ( self ) HoJ:ToggleShowHolyPowerFrame() end )
	checkShowHolyPowerFrame:Show()
	
	count = count + 2
	prev = checkShowHolyPowerFrame

	count, prev = HoJ:AddHeadingToScrollList( paladinCheckFrame, prev, "BOTTOMLEFT", 0, -8, 10, count, HoJ._text["Spec Toggles"] )

	local checkEnableRetribution = HoJ:SetupCheckBox( paladinCheckFrame, prev, 
			"checkEnableRetribution", "TOPLEFT", "BOTTOMLEFT", 0, 0, 
			HoJ[HoJ.CLASS_NAME_PALADIN][HoJ.SPEC_NAME_PALADIN_RETRIBUTION]._enabled, 10, HoJ._text["Enable for Retribution"] )
	checkEnableRetribution:SetScript( "OnClick", 
			function ( self ) HoJ:ToggleEnableForSpec( HoJ.CLASS_NAME_PALADIN, HoJ.SPEC_NAME_PALADIN_RETRIBUTION ) end )
	checkEnableRetribution:Show()

	local checkEnableProtPaladin = HoJ:SetupCheckBox( paladinCheckFrame, checkEnableRetribution, 
			"checkEnableProtPaladin", "TOPLEFT", "BOTTOMLEFT", 0, 0, 
			HoJ[HoJ.CLASS_NAME_PALADIN][HoJ.SPEC_NAME_PALADIN_PROTECTION]._enabled, 10, HoJ._text["Enable for Protection"] )
	checkEnableProtPaladin:SetScript( "OnClick", 
			function ( self ) HoJ:ToggleEnableForSpec( HoJ.CLASS_NAME_PALADIN, HoJ.SPEC_NAME_PALADIN_PROTECTION ) end )
	checkEnableProtPaladin:Show()

	count = count + 2
	prev = checkEnableProtPaladin

	count, prev = HoJ:AddHeadingToScrollList( paladinCheckFrame, prev, "BOTTOMLEFT", 0, -8, 10, count, HoJ._text["Skill Toggles"] )

	local anchor = "TOPLEFT"
	count, prev, anchor = HoJ:SetupCheckboxesForSpellInfos( HoJ[HoJ.CLASS_NAME_PALADIN]._spellInfos, paladinCheckFrame, prev, anchor, -16, count )

	-- set the scroll view height
	paladinScrollView.itemHeight = prev:GetHeight() - YPAD
	local itemsInPaladinScrollView = 8
	local totalHeight = ( paladinScrollView.itemHeight ) * ( count - itemsInPaladinScrollView )
	paladinCheckFrame:SetSize( 400, totalHeight )
	paladinScrollView:SetSize( 400, totalHeight )

	local paladinSlider = CreateFrame( "SLIDER", "paladinSlider", paladinScrollView );
	paladinSlider.name = "paladinSlider";
	paladinSlider:SetOrientation( "VERTICAL" );
	paladinSlider:SetHitRectInsets( 0, 0, -10, 0 );
	paladinSlider:SetBackdrop( sliderBackdrop );
	paladinSlider:SetWidth( SCROLLBAR_WIDTH );
	paladinSlider:SetThumbTexture( "Interface\\Buttons\\UI-SliderBar-Button-Vertical" );
	paladinSlider.scrollView = paladinScrollView;
	paladinSlider.slider = paladinSlider;
	paladinSlider:SetPoint( "TOPLEFT", paladinScrollView, "TOPRIGHT", -SCROLLBAR_WIDTH, 0 );
	paladinSlider:SetPoint( "BOTTOMLEFT", paladinScrollView, "BOTTOMRIGHT", -SCROLLBAR_WIDTH, 0 );
	paladinSlider:SetScript( "OnValueChanged", function ( slider, value ) HoJ:OnScrollValueChanged( slider, value ) end );
	paladinSlider:SetMinMaxValues( 0, totalHeight );
	paladinSlider:SetValueStep( 1 );
	paladinSlider:SetValue( 0 );
	paladinScrollView.slider = paladinSlider;

	------------------------------------------------------------
	-- warrior skills list
	------------------------------------------------------------
	local SCROLL_BORDER_HEIGHT = 24 * 8 + EDGE_SIZE * 2;
	local warriorScrollBorder = CreateFrame( "FRAME", "warriorScrollBorder", panel );
	warriorScrollBorder:SetPoint( "TOPLEFT", paladinScrollBorder, "TOPRIGHT", EDGE_SIZE, 0 );
	warriorScrollBorder:SetPoint( "TOPRIGHT", panel, "TOPRIGHT", -EDGE_SIZE, 0 );
	warriorScrollBorder:SetPoint( "BOTTOMLEFT", paladinScrollBorder, "BOTTOMRIGHT", EDGE_SIZE * 2, 0 );
	warriorScrollBorder:SetBackdrop( listBackdrop );

	local warriorChecklistTitle = panel:CreateFontString( nil, "ARTWORK" );
	warriorChecklistTitle:SetFont( "Fonts\\FRIZQT__.TTF", 12 );
	warriorChecklistTitle:SetJustifyH( "LEFT" );
	warriorChecklistTitle:SetJustifyV( "TOP" );
	warriorChecklistTitle:ClearAllPoints();
	warriorChecklistTitle:SetTextColor( 1, 1, 1, 1 );
	warriorChecklistTitle:SetPoint( "BOTTOMLEFT", warriorScrollBorder, "TOPLEFT", 0, 0 );
	warriorChecklistTitle:SetText( HoJ._text["Warrior Settings"] );		


	local warriorScrollView = CreateFrame( "SCROLLFRAME", "warriorScrollView", warriorScrollBorder );
	warriorScrollView:EnableMouseWheel( true );
	warriorScrollView:SetScript( "OnMouseWheel", scrollView_OnMouseWheel );
	warriorScrollView:SetPoint( "TOPLEFT", warriorScrollBorder, "TOPLEFT", EDGE_SIZE, -EDGE_SIZE );
	warriorScrollView:SetPoint( "TOPRIGHT", warriorScrollBorder, "TOPRIGHT", -EDGE_SIZE, -EDGE_SIZE );
	warriorScrollView:SetPoint( "BOTTOMLEFT", warriorScrollBorder, "BOTTOMLEFT", EDGE_SIZE,
	EDGE_SIZE );
	warriorScrollView:SetPoint( "BOTTOMRIGHT", warriorScrollBorder, "BOTTOMRIGHT", -EDGE_SIZE, EDGE_SIZE );

	local warriorCheckFrame = CreateFrame( "FRAME", "warriorItemFrame", warriorScrollView );
	warriorScrollView:SetScrollChild( warriorCheckFrame );

	warriorCheckFrame:SetPoint( "TOPLEFT", warriorScrollView, "TOPLEFT", 0, 0 );
	--warriorCheckFrame:SetPoint( "TOPRIGHT", warriorScrollView, "TOPRIGHT", -EDGE_SIZE, -EDGE_SIZE * 2 );
	--warriorCheckFrame:SetPoint( "BOTTOMLEFT", warriorScrollView, "BOTTOMLEFT", EDGE_SIZE, EDGE_SIZE );
	--warriorCheckFrame:SetPoint( "BOTTOMRIGHT", warriorScrollView, "BOTTOMRIGHT", -EDGE_SIZE, EDGE_SIZE );
	warriorCheckFrame:Lower();

	YPAD = 2;
	count = 0;
	local prev = nil

	count, prev = HoJ:AddHeadingToScrollList( warriorCheckFrame, warriorCheckFrame, "TOPLEFT", 0, 0, 10, count, HoJ._text["Spec Toggles"] )

	local checkEnableArms = HoJ:SetupCheckBox( warriorCheckFrame, prev, 
			"checkEnableArms", "TOPLEFT", "BOTTOMLEFT", 0, 0, 
			HoJ[HoJ.CLASS_NAME_WARRIOR][HoJ.SPEC_NAME_WARRIOR_ARMS]._enabled, 10, HoJ._text["Enable for Arms"] )
	checkEnableArms:SetScript( "OnClick", 
			function ( self ) HoJ:ToggleEnableForSpec( HoJ.CLASS_NAME_WARRIOR, HoJ.SPEC_NAME_WARRIOR_ARMS ) end )
	checkEnableArms:Show()

	local checkEnableFury = HoJ:SetupCheckBox( warriorCheckFrame, checkEnableArms, 
			"checkEnableFury", "TOPLEFT", "BOTTOMLEFT", 0, 0, 
			HoJ[HoJ.CLASS_NAME_WARRIOR][HoJ.SPEC_NAME_WARRIOR_FURY]._enabled, 10, HoJ._text["Enable for Fury"] );
	checkEnableFury:SetScript( "OnClick", 
			function ( self ) HoJ:ToggleEnableForSpec( HoJ.CLASS_NAME_WARRIOR, HoJ.SPEC_NAME_WARRIOR_FURY ) end )
	checkEnableFury:Show()

	local checkEnableProtWarrior = HoJ:SetupCheckBox( warriorCheckFrame, checkEnableFury, 
			"checkEnableProtWarrior", "TOPLEFT", "BOTTOMLEFT", 0, 0, 
			HoJ[HoJ.CLASS_NAME_WARRIOR][HoJ.SPEC_NAME_WARRIOR_PROTECTION]._enabled, 10, HoJ._text["Enable for Protection"] )
	checkEnableProtWarrior:SetScript( "OnClick", 
			function ( self ) HoJ:ToggleEnableForSpec( HoJ.CLASS_NAME_WARRIOR, HoJ.SPEC_NAME_WARRIOR_PROTECTION ) end )
	checkEnableProtWarrior:Show()

	count = count + 3
	prev = checkEnableProtWarrior

	count, prev = HoJ:AddHeadingToScrollList( warriorCheckFrame, prev, "BOTTOMLEFT", 0, -8, 10, count, HoJ._text["Skill Toggles"] )

	anchor = "TOPLEFT"
	count, prev, anchor = HoJ:SetupCheckboxesForSpellInfos( HoJ[HoJ.CLASS_NAME_WARRIOR]._spellInfos, warriorCheckFrame, prev, anchor, -16, count )

	-- There must be a better way to calculate the distance from the top of the first item to the bottom of the last item
	warriorScrollView.itemHeight = prev:GetHeight() - YPAD;
	local itemsInwarriorScrollView = 8;
	local totalHeight = ( warriorScrollView.itemHeight ) * ( count - itemsInwarriorScrollView );
	warriorCheckFrame:SetSize( 400, totalHeight );
	warriorScrollView:SetSize( 400, totalHeight );

	local warriorSlider = CreateFrame( "SLIDER", "warriorSlider", warriorScrollView );
	warriorSlider.name = "warriorSlider";
	warriorSlider:SetOrientation( "VERTICAL" );
	warriorSlider:SetHitRectInsets( 0, 0, -10, 0 );
	warriorSlider:SetBackdrop( sliderBackdrop );
	warriorSlider:SetWidth( SCROLLBAR_WIDTH );
	warriorSlider:SetThumbTexture( "Interface\\Buttons\\UI-SliderBar-Button-Vertical" );
	warriorSlider.scrollView = warriorScrollView;
	warriorSlider.slider = warriorSlider;
	warriorSlider:SetPoint( "TOPLEFT", warriorScrollView, "TOPRIGHT", -SCROLLBAR_WIDTH, 0 );
	warriorSlider:SetPoint( "BOTTOMLEFT", warriorScrollView, "BOTTOMRIGHT", -SCROLLBAR_WIDTH, 0 );
	warriorSlider:SetScript( "OnValueChanged", function ( slider, value ) HoJ:OnScrollValueChanged( slider, value ) end );
	warriorSlider:SetMinMaxValues( 0, totalHeight );
	warriorSlider:SetValueStep( 1 );
	warriorSlider:SetValue( 0 );
	warriorScrollView.slider = warriorSlider;

	------------------------------------------------------------
	-- frame toggles
	local frameToggleHeader = HoJ:AddTextToFrame( panel, paladinScrollBorder, "TOPLEFT", "BOTTOMLEFT", 0, -16, 1, 1, 1, 14, HoJ._text["Global Frame Toggles"] )

	-- add check box for skills bar
	local checkShowCooldownsFrame = HoJ:SetupCheckBox( panel, frameToggleHeader, "checkCooldowns", "TOPLEFT", "BOTTOMLEFT", 
			0, 0, HoJ._showCooldownsFrame, 11, "Show Cooldowns" );
			checkShowCooldownsFrame:SetScript( "OnClick", function ( self ) HoJ:ToggleShowSkillSFrame() end );
			checkShowCooldownsFrame:Show();

	-- add check box for buffs bar
	local checkShowBuffsFrame = HoJ:SetupCheckBox( panel, checkShowCooldownsFrame, "checkBuffs", "TOPLEFT", "TOPLEFT", 
			HoJ:GetCheckWidthWithText( checkShowCooldownsFrame ) + 16, 0, HoJ._showBuffsFrame, 11, "Show Buffs" );
			checkShowBuffsFrame:SetScript( "OnClick", function ( self ) HoJ:ToggleShowBuffsFrame() end );
			checkShowBuffsFrame:Show();

	local checkShowRotationFrame = HoJ:SetupCheckBox( panel, checkShowBuffsFrame, "checkRotation", "TOPLEFT", "TOPLEFT", 
			HoJ:GetCheckWidthWithText( checkShowBuffsFrame ) + 16, 0, HoJ._showRotationFrame, 11, "Show Rotation" );
			checkShowRotationFrame:SetScript( "OnClick", function ( self ) HoJ:ToggleShowRotationFrame() end );
			checkShowRotationFrame:Show();

	local checkShowRotationSecondaryFrame = HoJ:SetupCheckBox( panel, checkShowRotationFrame, "checkRotationSecondary", "TOPLEFT", "TOPLEFT", 
			HoJ:GetCheckWidthWithText( checkShowRotationFrame ) + 16, 0, HoJ._showRotationSecondaryFrame, 11, "Show Secondary Rotation" );
			checkShowRotationSecondaryFrame:SetScript( "OnClick", function ( self ) HoJ:ToggleShowRotationSecondaryFrame() end );
			checkShowRotationSecondaryFrame:Show();	
	
	local checkShowDebugFrame = HoJ:SetupCheckBox( panel, checkShowCooldownsFrame, "checkShowDebug", "TOPLEFT", "BOTTOMLEFT", 
			0, 0, HoJ._showDebugFrame, 11, "Show Debug Frame" );
			checkShowDebugFrame:SetScript( "OnClick", function ( self ) HoJ:ToggleShowDebugFrame() end );
			checkShowDebugFrame:Show();	

	local frameAlphaHeader = HoJ:AddTextToFrame( panel, checkShowDebugFrame, "TOPLEFT", "BOTTOMLEFT", 0, -16, 1, 1, 1, 14, HoJ._text["Frame Alpha"] )

	local alphaSlider = self:AddHorizontalSlider( "uiAlphaSlider", "Global:", panel, frameAlphaHeader,
			"BOTTOMLEFT", 0, -8, 250, 16, 1, 10, 10, "0.1", "1.0", Clamp( self._globalAlpha * 10, 1, 10 ),
			function ( alphaSlider, value )
				getglobal( alphaSlider:GetName() .. "Text" ):SetText( "Global: " .. string.format( "%.1f", value * 0.1 ) )
				HoJ:OnGlobalAlphaChanged( alphaSlider, value ) 
			end 
		)

	local buffsSlider = self:AddHorizontalSlider( "uiBuffsSlider", "Buffs:", panel, frameAlphaHeader,
			"BOTTOMLEFT", 0, -28, 250, 16, 1, 10, 10, "0.1", "1.0", Clamp( self._buffsAlpha * 10, 1, 10 ),
			function ( buffsSlider, value ) 
				getglobal( buffsSlider:GetName() .. "Text" ):SetText( "Buffs: " .. string.format( "%.1f", value * 0.1 ) )
				HoJ:OnBuffsAlphaChanged( buffsSlider, value ) 
			end 
		)

	local rotationSlider = self:AddHorizontalSlider( "uiRotationSlider", "Rotation:", panel, frameAlphaHeader,
			"BOTTOMLEFT", 0, -48, 250, 16, 1, 10, 10, "0.1", "1.0", Clamp( self._rotationAlpha * 10, 1, 10 ),
			function ( rotationSlider, value ) 
				getglobal( rotationSlider:GetName() .. "Text" ):SetText( "Rotation: " .. string.format( "%.1f", value * 0.1 ) )
				HoJ:OnRotationAlphaChanged( rotationSlider, value ) 
			end 
		)

	local rotationSecondarySlider = self:AddHorizontalSlider( "uiRotationSecondarySlider", "Rotation Secondary:", panel, frameAlphaHeader,
			"BOTTOMLEFT", 0, -68, 250, 16, 1, 10, 10, "0.1", "1.0", Clamp( self._rotationSecondaryAlpha * 10, 1, 10 ),
			function ( rotationSecondarySlider, value ) 
				getglobal( rotationSecondarySlider:GetName() .. "Text" ):SetText( "Rotation Secondary: " .. string.format( "%.1f", value * 0.1 ) )
				HoJ:OnRotationSecondaryAlphaChanged( rotationSecondarySlider, value ) 
			end 
		)

	local cooldownsSlider = self:AddHorizontalSlider( "uiCooldownsSlider", "Cooldowns:", panel, frameAlphaHeader,
			"BOTTOMLEFT", 0, -88, 250, 16, 1, 10, 10, "0.1", "1.0", Clamp( self._cooldownsAlpha * 10, 1, 10 ),
			function ( cooldownsSlider, value ) 
				getglobal( cooldownsSlider:GetName() .. "Text" ):SetText( "Cooldowns: " .. string.format( "%.1f", value * 0.1 ) )
				HoJ:OnCooldownsAlphaChanged( cooldownsSlider, value ) 
			end 
		)

	local hopoSlider = self:AddHorizontalSlider( "uiHoPoSlider", "Holy Power:", panel, frameAlphaHeader,
			"BOTTOMLEFT", 0, -108, 250, 16, 1, 10, 10, "0.1", "1.0", Clamp( self._holyPowerAlpha * 10, 1, 10 ),
			function ( hopoSlider, value ) 
				getglobal( hopoSlider:GetName() .. "Text" ):SetText( "Holy Power: " .. string.format( "%.1f", value * 0.1 ) )
				HoJ:OnHolyPowerAlphaChanged( hopoSlider, value ) 
			end 
		)

	local debugSlider = self:AddHorizontalSlider( "uiDebugSlider", "Debug:", panel, frameAlphaHeader,
			"BOTTOMLEFT", 0, -128, 250, 16, 1, 10, 10, "0.1", "1.0", Clamp( self._debugAlpha * 10, 1, 10 ),
			function ( debugSlider, value ) 
				getglobal( debugSlider:GetName() .. "Text" ):SetText( "Debug: " .. string.format( "%.1f", value * 0.1 ) )
				HoJ:OnDebugAlphaChanged( debugSlider, value ) 
			end 
		)


	paladinScrollView:Show()
	paladinCheckFrame:Show()
	warriorScrollView:Show()
	warriorCheckFrame:Show()
	paladinSlider:Show()
	warriorSlider:Show()
	alphaSlider:Show()
end

